#include "mex.h"
#include "Mates.h"
#include "Debugging.h"

#include <stdio.h>
#include <math.h>
#include <time.h>
#include <float.h>
#include <string.h>


/* 
In order to compile this MEX function, type the following at the MATLAB prompt:
32-bit Windows:
mex WinnerTSOMMEX.c MatesLap.c lapack.a blas.a libf2c.a Debugging.c
64-bit Windows:
mex LINKFLAGS="$LINKFLAGS /NODEFAULTLIB:LIBCMT" WinnerTSOMMEX.c MatesLap.c Debugging.c clapack_nowrap.lib BLAS_nowrap.lib libf2c.lib mpir.lib mpfr.lib


Usage:
[Winners] = WinnerTSOMMEX(Model,Samples);



*/

/* Natural logarithm of the gamma function */
double gammln(double argument);


void mexFunction(int nlhs, mxArray* plhs[],
                 int nrhs, const mxArray* prhs[])
{  
  
    
    const int *DimMap;
    const int *DimSamples;
    int SpaceDimension,NumRowsMap,NumColsMap,Index,NumSamples;
    int NdxCol,NdxRow,NdxSample,NdxDim;
    mxArray *Means,*C,*Nu,*CInv;
    double *ptrVectorDif,*ptrVectorProd,*ptrMyMean,*ptrMyC,*ptrMyNu,*ptrMyCInv;
    double *ptrMyEigenValues,*ptrMyEigenVectors,*ptrSamples,*ptrMySample;
    double *ptrLogDetC,*ptrWinners;
    double MyLogDensity,D,MyNu,BestLogDensity,DistMahal;

    
    /* Get input data */
    Means=mxGetField(prhs[0],0,"Means");
    C=mxGetField(prhs[0],0,"C");
    CInv=mxGetField(prhs[0],0,"CInv");
    Nu=mxGetField(prhs[0],0,"Nu");
    DimMap=mxGetDimensions(Nu);
    NumRowsMap=DimMap[0];
    NumColsMap=DimMap[1];

    DimSamples=mxGetDimensions(prhs[1]);
    SpaceDimension=DimSamples[0];
    NumSamples=DimSamples[1];
    ptrSamples=mxGetPr(prhs[1]);
    
    /* Create output matrix */
    plhs[0]=mxCreateDoubleMatrix(NumSamples,1,mxREAL);
    ptrWinners=mxGetPr(plhs[0]);
    
    /* Create auxiliary matrices */
    ptrVectorDif=mxMalloc(SpaceDimension*1*sizeof(double));
    ptrVectorProd=mxMalloc(SpaceDimension*1*sizeof(double));
    ptrMyEigenValues=mxMalloc(SpaceDimension*1*sizeof(double));
    ptrMyEigenVectors=mxMalloc(SpaceDimension*SpaceDimension*sizeof(double));   
    ptrLogDetC=mxMalloc(NumRowsMap*NumColsMap*sizeof(double));
    
    /* Find the eigenvalues of the covariance matrices */
    Index=0;
    for(NdxCol=0;NdxCol<NumColsMap;NdxCol++)
    {
        for(NdxRow=0;NdxRow<NumRowsMap;NdxRow++)
        {
            /* Prepare working variables */
            ptrMyC=mxGetPr(mxGetCell(C,Index));
            
            /* Find the eigenvalues in order to compute the determinant of the covariance matrix */
            EigValVec(ptrMyC,ptrMyEigenValues,ptrMyEigenVectors,SpaceDimension);    
            
            /* Finding log(det C) */
            ptrLogDetC[Index]=0.0;
            for(NdxDim=0;NdxDim<SpaceDimension;NdxDim++)
            {
                (ptrLogDetC[Index])+=log(ptrMyEigenValues[NdxDim]);
            }
            
            Index++;        
        }
    }
    
    /* Compute the winner for all the samples */
    for(NdxSample=0;NdxSample<NumSamples;NdxSample++)
    {
        ptrMySample=ptrSamples+NdxSample*SpaceDimension;
        Index=0;
        /* BestLogDensity = -inf; */
        BestLogDensity=-DBL_MAX;
        
        for(NdxCol=0;NdxCol<NumColsMap;NdxCol++)
        {
            for(NdxRow=0;NdxRow<NumRowsMap;NdxRow++)
            {
                /* Prepare working variables */
                ptrMyMean=mxGetPr(mxGetCell(Means,Index));
                ptrMyC=mxGetPr(mxGetCell(C,Index));
                ptrMyCInv=mxGetPr(mxGetCell(CInv,Index));
                ptrMyNu=mxGetPr(mxGetCell(Nu,Index));
                
                
                /* VectorDif=Samples(:,NdxSample) - Model.Mu{NdxRow,NdxCol}; */
                MatrixDifference(ptrMySample,ptrMyMean,
                        ptrVectorDif,SpaceDimension,1);
                
                
                /* Find the squared Mahalanobis distance with respect to the covariance matrix */
                MatrixProduct(ptrMyCInv,ptrVectorDif,ptrVectorProd,SpaceDimension,
                        SpaceDimension,1);
                MatrixProduct(ptrVectorDif,ptrVectorProd,&DistMahal,
                        1,SpaceDimension,1);
                
                                                             
                /* Finding log(p(t sub n | i) */
                D=(double)SpaceDimension;
                MyNu=(*ptrMyNu);
                
                MyLogDensity=gammln(0.5*D+0.5*MyNu)-gammln(0.5*MyNu)-0.5*ptrLogDetC[Index]
                     -0.5*D*(1.144729885849400+log(MyNu-2.0))
                     -0.5*(D+MyNu)*log(1.0+DistMahal/(MyNu-2.0));
                
                if (BestLogDensity<MyLogDensity)
                {
                     BestLogDensity=MyLogDensity;
                     ptrWinners[NdxSample]=Index+1;
                                                   
                }
                
                /* Index to iterate through the cells */
                Index++;
         
            }    
     
        } 
    }   
    
    
    
    
    /* Release memory */
    
    mxFree(ptrVectorDif);
    mxFree(ptrVectorProd);
    mxFree(ptrMyEigenValues);
    mxFree(ptrMyEigenVectors); 
    mxFree(ptrLogDetC);
    
    
}    



/*

  Purpose:

    ALNGAM computes the logarithm of the gamma function.

  Modified:

    13 January 2008

  Author:

    Allan Macleod
    C++ version by John Burkardt

  Reference:

    Allan Macleod,
    Algorithm AS 245,
    A Robust and Reliable Algorithm for the Logarithm of the Gamma Function,
    Applied Statistics,
    Volume 38, Number 2, 1989, pages 397-402.

  Parameters:

    Input, double XVALUE, the argument of the Gamma function.

    Output, int IFAULT, error flag.
    0, no error occurred.
    1, XVALUE is less than or equal to 0.
    2, XVALUE is too big.

    Output, double ALNGAM, the logarithm of the gamma function of X.
*/
double alngam ( double xvalue, int *ifault )
{
  double alr2pi = 0.918938533204673;
  double r1[9] = {
    -2.66685511495, 
    -24.4387534237, 
    -21.9698958928, 
     11.1667541262, 
     3.13060547623, 
     0.607771387771, 
     11.9400905721, 
     31.4690115749, 
     15.2346874070 };
  double r2[9] = {
    -78.3359299449, 
    -142.046296688, 
     137.519416416, 
     78.6994924154, 
     4.16438922228, 
     47.0668766060, 
     313.399215894, 
     263.505074721, 
     43.3400022514 };
  double r3[9] = {
    -2.12159572323E+05, 
     2.30661510616E+05, 
     2.74647644705E+04, 
    -4.02621119975E+04, 
    -2.29660729780E+03, 
    -1.16328495004E+05, 
    -1.46025937511E+05, 
    -2.42357409629E+04, 
    -5.70691009324E+02 };
  double r4[5] = {
     0.279195317918525, 
     0.4917317610505968, 
     0.0692910599291889, 
     3.350343815022304, 
     6.012459259764103 };
  double value;
  double x;
  double x1;
  double x2;
  double xlge = 510000.0;
  double xlgst = 1.0E+30;
  double y;

  x = xvalue;
  value = 0.0;
/*
  Check the input.
*/
  if ( xlgst <= x )
  {
    *ifault = 2;
    return value;
  }

  if ( x <= 0.0 )
  {
    *ifault = 1;
    return value;
  }

  *ifault = 0;
/*
  Calculation for 0 < X < 0.5 and 0.5 <= X < 1.5 combined.
*/
  if ( x < 1.5 )
  {
    if ( x < 0.5 )
    {
      value = - log ( x );
      y = x + 1.0;
/*
  Test whether X < machine epsilon.
*/
      if ( y == 1.0 )
      {
        return value;
      }
    }
    else
    {
      value = 0.0;
      y = x;
      x = ( x - 0.5 ) - 0.5;
    }

    value = value + x * (((( 
        r1[4]   * y 
      + r1[3] ) * y 
      + r1[2] ) * y 
      + r1[1] ) * y 
      + r1[0] ) / (((( 
                  y 
      + r1[8] ) * y 
      + r1[7] ) * y 
      + r1[6] ) * y 
      + r1[5] );

    return value;
  }
/*
  Calculation for 1.5 <= X < 4.0.
*/
  if ( x < 4.0 )
  {
    y = ( x - 1.0 ) - 1.0;

    value = y * (((( 
        r2[4]   * x 
      + r2[3] ) * x 
      + r2[2] ) * x 
      + r2[1] ) * x 
      + r2[0] ) / (((( 
                  x 
      + r2[8] ) * x 
      + r2[7] ) * x 
      + r2[6] ) * x 
      + r2[5] );
  }
/*
  Calculation for 4.0 <= X < 12.0.
*/
  else if ( x < 12.0 ) 
  {
    value = (((( 
        r3[4]   * x 
      + r3[3] ) * x 
      + r3[2] ) * x 
      + r3[1] ) * x 
      + r3[0] ) / (((( 
                  x 
      + r3[8] ) * x 
      + r3[7] ) * x 
      + r3[6] ) * x 
      + r3[5] );
  }
/*
  Calculation for 12.0 <= X.
*/
  else
  {
    y = log ( x );
    value = x * ( y - 1.0 ) - 0.5 * y + alr2pi;

    if ( x <= xlge )
    {
      x1 = 1.0 / x;
      x2 = x1 * x1;

      value = value + x1 * ( ( 
             r4[2]   * 
        x2 + r4[1] ) * 
        x2 + r4[0] ) / ( ( 
        x2 + r4[4] ) * 
        x2 + r4[3] );
    }
  }

  return value;
}

/* Natural logarithm of the gamma function */
double gammln(double argument)
{
	int ifault;
    
    return alngam(argument,&ifault);

}

